package cn.kdan.compdf.config;

import cn.kdan.compdf.dto.LoginUser;
import cn.kdan.compdf.dto.LoginUserDTO;
import cn.kdan.compdf.enums.AuthEnum;
import cn.kdan.compdf.enums.JwtTokenEnum;
import cn.kdan.compdf.exception.BusinessException;
import lombok.SneakyThrows;
import org.springframework.stereotype.Component;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jws;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.security.Keys;

import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.util.Base64;
import java.util.Date;

/**
 * @author Bob 2022-10-11
 **/
@Component
public class JwtConfig {

    /**
     * 加密
     */
    public static String generateJwt(LoginUserDTO user) {

        // 加密
        byte[] keyBytes = AuthEnum.SECRET.getBytes();
        // 获得密钥对象
        SecretKey key = Keys.hmacShaKeyFor(keyBytes);

        String token = Jwts.builder()
                .setHeaderParam("typ", "JWT") //令牌类型
                .setIssuedAt(new Date()) //签发时间
                .setExpiration(new Date(System.currentTimeMillis() + AuthEnum.EXPIRE * 1000 * 60 * 60)) //过期时间
                .claim(JwtTokenEnum.ID, user.getId())
                .claim(JwtTokenEnum.USERNAME, user.getUsername())
                .claim(JwtTokenEnum.FULL_NAME, user.getFullName())
                .claim(JwtTokenEnum.TENANT_ID, user.getTenantId())
                .signWith(key, SignatureAlgorithm.HS256)
                .compact();
        return token;
    }

    /**
     * 解析jwt，一定要捕获异常
     * @param jwtToken
     * @return
     */
    public static LoginUser checkJwt(String jwtToken) {
        Jws<Claims> claimsJws = Jwts.parser().setSigningKey(AuthEnum.SECRET.getBytes()).parseClaimsJws(jwtToken);
        // map
        Claims claims = claimsJws.getBody();
        Long id = claims.get(JwtTokenEnum.ID,Long.class);
        String email = claims.get(JwtTokenEnum.EMAIL, String.class);
        return LoginUser.builder()
                .username(email)
                .compdfkitId(id)
                .build();
    }

    @SneakyThrows
   public static void main(String[] args) {
//        String s = generateJwt(LoginUserDTO.builder().id(123L).username("kdan").tenantId("1").fullName("kk").build());
//        System.out.println(s);
       // 加密

       try {
            String token = "eyJhbGciOiJIUzI1NiIsInR5cGUiOiJKV1QifQ.eyJlbWFpbCI6InBlbmdqaWFueW9uZ0BrZGFubW9iaWxlLmNvbSIsImlhdCI6MTY3MzMzNjY2NSwianRpIjoiMTc4ZmNiYWU5NmIxNDRhNzM1Yzg2ZWM0ZWNmNzYyODAiLCJpZCI6Nzh9.hhtkFLfJnmEsOCASgBlwuG7dwhr1iYne4GC433EG4vA";
//           LoginUserDTO loginUserDTO = checkJwt(token);
//           System.out.println(loginUserDTO);
//            System.out.println(loginUserDTO);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("登录过期");
        }
    }

}
