package cn.kdan.compdf.controller.v1;

import cn.kdan.compdf.base.R;
import cn.kdan.compdf.dto.DashboardStatisticsDTO;
import cn.kdan.compdf.entity.dto.AnalysisDataDTO;
import cn.kdan.compdf.entity.dto.DashboardDetailedDataDTO;
import cn.kdan.compdf.entity.dto.DashboardQueryDTO;
import cn.kdan.compdf.enums.AuthEnum;
import cn.kdan.compdf.exception.CommonException;
import cn.kdan.compdf.service.BackgroundConvertDataService;
import cn.kdan.compdf.service.BackgroundToolsService;
import cn.kdan.compdf.service.BackgroundUserProjectService;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.ProjectQueryVO;
import cn.kdan.compdf.vo.ToolQueryVO;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author comPDF-Kit WPH 2022-10-09
 */
@RestController
@RequestMapping("/v1/convert-data")
@RequiredArgsConstructor
public class BackgroundDashboardController {

    private final BackgroundConvertDataService backgroundConvertDataService;
    private final BackgroundUserProjectService backgroundUserProjectService;
    private final BackgroundToolsService toolsService;

    @Value("${background.dashboard.autograph}")
    private String DASHBOARD_SYNC_AUTOGRAPH;

    /**
     * 获取统计信息
     */
    @GetMapping("/getAnalysisData-byDay")
    public R<AnalysisDataDTO> getAnalysisData(@Validated DashboardQueryDTO dashboardQueryDTO, HttpServletRequest request) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        AnalysisDataDTO analysisData = backgroundConvertDataService.getAnalysisData(dashboardQueryDTO);
        return R.ok(analysisData);
    }

    /**
     * 获取成功数时间节点数据集合
     */
    @GetMapping("/getSuccessfulRequestDetailedData")
    public R<List<DashboardDetailedDataDTO>> getSuccessfulRequestDetailedData(@Validated DashboardQueryDTO dashboardQueryDTO, HttpServletRequest request) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        List<DashboardDetailedDataDTO> successfulRequestDetailedData = backgroundConvertDataService.getSuccessfulRequestDetailedData(dashboardQueryDTO);
        return R.ok(successfulRequestDetailedData);
    }

    /**
     * 获取失败数时间节点数据集合
     */
    @GetMapping("/getErrorRequestDetailedData")
    public R<List<DashboardDetailedDataDTO>> getErrorRequestDetailedData(@Validated DashboardQueryDTO dashboardQueryDTO, HttpServletRequest request) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        List<DashboardDetailedDataDTO> errorRequestDetailedData = backgroundConvertDataService.getErrorRequestDetailedData(dashboardQueryDTO);
        return R.ok(errorRequestDetailedData);
    }

    /**
     * 获取失败率时间节点数据集合
     */
    @GetMapping("/getErrorRatioDetailedData")
    public R<List<DashboardDetailedDataDTO>> getErrorRatioDetailedData(@Validated DashboardQueryDTO dashboardQueryDTO, HttpServletRequest request) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        List<DashboardDetailedDataDTO> errorRatioDetailedData = backgroundConvertDataService.getErrorRatioDetailedData(dashboardQueryDTO);
        return R.ok(errorRatioDetailedData);
    }

    /**
     * 获取转档时长时间节点数据集合
     */
    @GetMapping("/getAverageProcessTimeDetailedData")
    public R<List<DashboardDetailedDataDTO>> getAverageProcessTimeDetailedData(@Validated DashboardQueryDTO dashboardQueryDTO, HttpServletRequest request) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        List<DashboardDetailedDataDTO> averageProcessTimeDetailedData = backgroundConvertDataService.getAverageProcessTimeDetailedData(dashboardQueryDTO);
        return R.ok(averageProcessTimeDetailedData);
    }

    @GetMapping("/export")
    public void exportDashboard(@Validated DashboardQueryDTO dashboardQueryDTO,HttpServletRequest request, HttpServletResponse response) {
        dashboardQueryDTO.setTenantId(new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER)));
        backgroundConvertDataService.dashboardExport(dashboardQueryDTO, response);
    }

    /**
     * dashboard 数据同步接收
     */
    @PostMapping("/dashboardsync")
    public R<Void> dashboardsync(@RequestBody List<DashboardStatisticsDTO> dashboardStatisticsDTOList, HttpServletRequest request) {
        if (!DASHBOARD_SYNC_AUTOGRAPH.equals(request.getHeader("autograph"))) {
            throw new CommonException("错误");
        }
        backgroundConvertDataService.dashboardDataSync(dashboardStatisticsDTOList);
        return R.ok();
    }

    /**
     * 刷新当前用户的数据
     */
    @PostMapping("/refresh")
    public void test(HttpServletRequest request) {
        Long tenantId = new Long(request.getHeader(AuthEnum.USER_TENANT_ID_HEADER));
        backgroundConvertDataService.refresh(tenantId);
    }

    /**
     * 获取project查询使用
     */
    @GetMapping("/getProjects")
    public R<List<ProjectQueryVO>> getProjects(){
        return R.ok(backgroundUserProjectService.getProjectQuery(TokenUtil.getRequestHeader().getTenantId()));
    }

    /**
     * 获取tools查询使用
     */
    @GetMapping("/getTools")
    public R<List<ToolQueryVO>> getTools(){
        return R.ok(toolsService.getToolQueryS());
    }
}
