package cn.kdan.compdf.controller.v1;

import cn.hutool.core.util.ObjectUtil;
import cn.kdan.compdf.base.R;
import cn.kdan.compdf.dto.AddNewProjectDTO;
import cn.kdan.compdf.dto.EditProjectDTO;
import cn.kdan.compdf.enums.ResponseEnum;
import cn.kdan.compdf.exception.BusinessException;
import cn.kdan.compdf.exception.CommonException;
import cn.kdan.compdf.service.BackgroundUserProjectService;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.BackgroundUserProjectVO;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * API keys接口
 * @author comPDF-Kit WPH 2022-10-09
 */
@RestController
@RequestMapping("/v1/project")
public class BackgroundUserProjectController {

    private final BackgroundUserProjectService backgroundUserProjectService;

    public BackgroundUserProjectController(BackgroundUserProjectService backgroundUserProjectService) {
        this.backgroundUserProjectService = backgroundUserProjectService;
    }

    /**
     * 获取API Keys列表
     */
    @GetMapping("/getProjectList")
    public R<List<BackgroundUserProjectVO>> getProjectList(Integer timeZone) {
        if (ObjectUtil.isEmpty(timeZone)){
            timeZone = 0;
        }
        return R.ok(backgroundUserProjectService.getProjectList(TokenUtil.getRequestHeader().getId(),timeZone));
    }

    /**
     * 添加新项目
     */
    @PostMapping("/addNewProject")
    public R<Void> addNewProject(@Validated @RequestBody AddNewProjectDTO addNewProjectDTO) {
        if (ObjectUtil.isEmpty(addNewProjectDTO.getProjectName().trim())) {
            throw new CommonException(ResponseEnum.PROJECT_NULL);
        }

        addNewProjectDTO.setUserId(TokenUtil.getRequestHeader().getId());
        addNewProjectDTO.setTenantId(TokenUtil.getRequestHeader().getTenantId());
        backgroundUserProjectService.addNewProject(addNewProjectDTO);
        return R.ok();
    }

    /**
     * 添加新用户默认项目接口
     */
    @PostMapping("/defaultProject/{userId}")
    public R<Void> defaultProject(@PathVariable("userId") Long userId) {
        backgroundUserProjectService.defaultProject(userId,null);
        return R.ok();
    }

    /**
     * 编辑项目
     */
    @PutMapping("editProject")
    public R<Void> editProject(@Validated @RequestBody EditProjectDTO editProjectDTO) {
        backgroundUserProjectService.editProject(editProjectDTO,TokenUtil.getRequestHeader().getId());
        return R.ok();
    }

    /**
     * 删除项目
     */
    @DeleteMapping("/deleteById/{id}")
    public R<Void> deleteById(@PathVariable("id") Long id) {
        if (null == id) {
            throw new BusinessException("参数错误");
        }
        backgroundUserProjectService.deleteById(id);
        return R.ok();
    }
}
